@@ -1,13 +1,13 @@
 ;;; vulpea-ui.el --- Sidebar infrastructure and widget framework for vulpea notes -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2024-2025 Boris Buliga
+;; Copyright (C) 2024-2026 Boris Buliga
 ;; SPDX-License-Identifier: GPL-3.0-or-later
 
 ;; Author: Boris Buliga <boris@d12frosted.io>
 ;; Maintainer: Boris Buliga <boris@d12frosted.io>
 ;; URL: https://github.com/d12frosted/vulpea-ui
-;; Package-Version: 1.0.0
-;; Package-Revision: v1.0.0-0-g39c842e47bd5
+;; Package-Version: 1.1.0
+;; Package-Revision: v1.1.0-0-gfdd6749cb4a2
 ;; Package-Requires: ((emacs "29.1") (vulpea "2.0.0") (vui "0.1"))
 ;; Keywords: outlines hypermedia
 
@@ -54,6 +54,7 @@
 (require 'org-element)
 (require 'vulpea)
 (require 'vui)
+(require 'vui-components)
 
 
 ;;; Custom variables
@@ -106,11 +107,6 @@ When non-nil, shows a snippet of text around each backlink mention."
   :type 'boolean
   :group 'vulpea-ui)
 
-(defcustom vulpea-ui-backlinks-preview-lines 2
-  "Number of lines to show in backlink previews for prose context."
-  :type 'integer
-  :group 'vulpea-ui)
-
 (defcustom vulpea-ui-backlinks-prose-chars-before 30
   "Number of characters to show before link in prose previews."
   :type 'integer
@@ -313,7 +309,8 @@ Widgets are filtered by predicate and sorted by order."
   "Major mode for the vulpea-ui sidebar buffer.
 \\{vulpea-ui-sidebar-mode-map}"
   :group 'vulpea-ui
-  (setq-local truncate-lines t))
+  (setq-local truncate-lines t)
+  (mode-line-invisible-mode 1))
 
 
 ;;; Sidebar state (frame-local)
@@ -335,7 +332,7 @@ Used to prevent re-entry during render.")
   "Return the sidebar buffer name for FRAME.
 If FRAME is nil, use the selected frame."
   (let ((frame (or frame (selected-frame))))
-    (format "*vulpea-ui-sidebar:%s*" (frame-parameter frame 'name))))
+    (format "*vulpea-ui-sidebar:%s*" (or (frame-parameter frame 'window-id) ""))))
 
 (defun vulpea-ui--get-sidebar-buffer (&optional frame)
   "Get the sidebar buffer for FRAME, or nil if it doesn't exist."
@@ -499,14 +496,21 @@ Called from `window-buffer-change-functions'."
     (vulpea-ui-sidebar-refresh)))
 
 (defun vulpea-ui--on-idle ()
-  "Handle idle timeout - refresh stats and outline."
+  "Handle idle timeout - soft refresh preserving memos.
+Only widgets whose deps changed (e.g. `buffer-modified-tick' for
+stats and outline) will recompute."
   (when (vulpea-ui--sidebar-visible-p)
     (let* ((frame (selected-frame))
            (main-win (vulpea-ui--get-main-window frame))
            (main-buf (when main-win (window-buffer main-win)))
-           (note (vulpea-ui--get-note-from-buffer main-buf)))
-      (when note
-        (vulpea-ui-sidebar-refresh)))))
+           (note (vulpea-ui--get-note-from-buffer main-buf))
+           (instance (gethash frame vulpea-ui--sidebar-instances)))
+      (when (and note instance
+                 (vui-instance-buffer instance)
+                 (buffer-live-p (vui-instance-buffer instance)))
+        (with-current-buffer (vui-instance-buffer instance)
+          (setq vulpea-ui--current-note note))
+        (vui-update-props instance (list :note note))))))
 
 
 ;;; Utility functions
@@ -587,9 +591,7 @@ For use within widget components."
 (defun vulpea-ui-widget-toggle-at-point ()
   "Toggle the widget collapse state at point."
   (interactive)
-  (let ((widget (widget-at (point))))
-    (when widget
-      (widget-apply widget :action))))
+  (vulpea-ui-follow-link-at-point))
 
 
 ;;; Widget wrapper component
@@ -599,27 +601,17 @@ For use within widget components."
 TITLE is the widget title string.
 COUNT is an optional count to display in the header.
 CHILDREN (implicit) is a function returning the widget content."
-  :state ((collapsed vulpea-ui-default-widget-collapsed))
   :render
-  (vui-vstack
-   :spacing 0
-   ;; Header
-   (vui-hstack
-    :spacing 1
-    (vui-button (if collapsed "▶" "▼")
-      :on-click (lambda () (vui-set-state :collapsed (not collapsed)))
-      :face 'vulpea-ui-widget-header-face
-      :help-echo nil)
-    (vui-text title :face 'vulpea-ui-widget-header-face)
-    (when count
-      (vui-text (format "(%s)" count) :face 'vulpea-ui-widget-count-face)))
-   ;; Content (when not collapsed)
-   (unless collapsed
-     (when children
-       (vui-vstack
-        :spacing 0
-        :indent 2
-        (funcall children))))))
+  (let ((display-title (if count
+                           (format "%s (%s)" title count)
+                         title)))
+    (vui-collapsible
+      :title display-title
+      :initially-expanded (not vulpea-ui-default-widget-collapsed)
+      :title-face 'vulpea-ui-widget-header-face
+      :indent 2
+      (when children
+        (funcall children)))))
 
 
 ;;; Shared components
@@ -691,7 +683,10 @@ STRIP-METADATA removes org metadata lines when non-nil."
   :render
   (let ((note (use-vulpea-ui-note)))
     (when note
-      (let* ((stats (vui-use-memo (note)
+      (let* ((note-buf (when (vulpea-note-path note)
+                         (find-buffer-visiting (vulpea-note-path note))))
+             (tick (when note-buf (buffer-modified-tick note-buf)))
+             (stats (vui-use-memo (note tick)
                       (vulpea-ui--compute-stats note)))
              (chars (plist-get stats :chars))
              (words (plist-get stats :words))
@@ -751,8 +746,11 @@ Otherwise reads from disk."
   :render
   (let ((note (use-vulpea-ui-note)))
     (when note
-      (let ((headings (vui-use-memo (note)
-                        (vulpea-ui--parse-headings note))))
+      (let* ((note-buf (when (vulpea-note-path note)
+                         (find-buffer-visiting (vulpea-note-path note))))
+             (tick (when note-buf (buffer-modified-tick note-buf)))
+             (headings (vui-use-memo (note tick)
+                         (vulpea-ui--parse-headings note))))
         (vui-component 'vulpea-ui-widget
           :title "Outline"
           :count (length headings)
@@ -763,7 +761,7 @@ Otherwise reads from disk."
                  (lambda (heading)
                    (vulpea-ui--render-outline-heading heading note))
                  headings)
-              (vui-text "No headings" :face 'shadow))))))))
+              (vui-muted "No headings"))))))))
 
 (defun vulpea-ui--heading-archived-p (hl archive-tag)
   "Return non-nil if HL or any of its ancestors has ARCHIVE-TAG."
@@ -780,9 +778,13 @@ Otherwise reads from disk."
 Returns a list of plists with :title, :level, and :pos."
   (when (and note (vulpea-note-path note))
     (let ((path (vulpea-note-path note))
-          (max-depth vulpea-ui-outline-max-depth))
+          (max-depth vulpea-ui-outline-max-depth)
+          (existing-buf (find-buffer-visiting (vulpea-note-path note))))
       (with-temp-buffer
-        (insert-file-contents path)
+        (if existing-buf
+            (insert (with-current-buffer existing-buf
+                      (buffer-substring-no-properties (point-min) (point-max))))
+          (insert-file-contents path))
         (vulpea-ui--setup-org-mode)
         (let ((headings nil)
               (archive-tag org-archive-tag))
@@ -837,19 +839,20 @@ Groups backlinks by file and shows heading context with optional previews."
                        (vulpea-ui--get-grouped-backlinks note)))
              (groups (plist-get result :groups))
              (filtered-count (plist-get result :filtered-count))
-             (total-count (plist-get result :total-count)))
+             (total-count (plist-get result :total-count))
+             (count-display (if (= filtered-count total-count)
+                                filtered-count
+                              (format "%d/%d" filtered-count total-count))))
         (vui-component 'vulpea-ui-widget
           :title "Backlinks"
-          :count (if (= filtered-count total-count)
-                     filtered-count
-                   (format "%d/%d" filtered-count total-count))
+          :count count-display
           :children
           (lambda ()
             (if groups
                 (vui-vstack
                  :spacing 1
                  (seq-map #'vulpea-ui--render-backlink-group groups))
-              (vui-text "No backlinks" :face 'shadow))))))))
+              (vui-muted "No backlinks"))))))))
 
 (defun vulpea-ui--get-grouped-backlinks (note)
   "Get backlinks to NOTE grouped by file.
@@ -1206,7 +1209,7 @@ This is an internal wrapper around `vulpea-ui-clean-org-markup'."
      ;; File-level note link
      (if file-note
          (vui-component 'vulpea-ui-note-link :note file-note)
-       (vui-text (file-name-nondirectory path) :face 'shadow))
+       (vui-muted (file-name-nondirectory path)))
      ;; Mentions grouped by heading
      (when grouped
        (vui-vstack
@@ -1268,57 +1271,33 @@ Renders the preview as a clickable button to jump to the mention."
   "Render PREVIEW as a clickable button to jump to PATH at POS."
   (let ((type (plist-get preview :type))
         (on-click (lambda () (vulpea-ui--jump-to-file-position path pos))))
-    (pcase type
-      ('meta
-       (vui-button (concat "⊢ " (plist-get preview :key) ": "
-                           (or (plist-get preview :value) ""))
-         :face 'vulpea-ui-backlink-preview-face
-         :no-decoration t
-         :on-click on-click
-         :help-echo nil))
-      ('header
-       ;; Header context means the link is IN the heading text - make it clickable
-       (vui-button (concat "§ " (plist-get preview :text))
-         :face 'vulpea-ui-backlink-preview-face
-         :no-decoration t
-         :on-click on-click
-         :help-echo nil))
-      ('table
-       (vui-button (concat "▤ " (plist-get preview :text))
-         :face 'vulpea-ui-backlink-preview-face
-         :no-decoration t
-         :on-click on-click
-         :help-echo nil))
-      ('list
-       (vui-button (concat "· " (plist-get preview :text))
-         :face 'vulpea-ui-backlink-preview-face
-         :no-decoration t
-         :on-click on-click
-         :help-echo nil))
-      ('quote
-       (vui-button (concat "> " (plist-get preview :text))
-         :face 'vulpea-ui-backlink-preview-face
-         :no-decoration t
-         :on-click on-click
-         :help-echo nil))
-      ('code
-       (vui-button (concat "λ " (plist-get preview :text))
-         :face 'vulpea-ui-backlink-preview-face
-         :no-decoration t
-         :on-click on-click
-         :help-echo nil))
-      ('footnote
-       (vui-button (concat "† " (plist-get preview :text))
-         :face 'vulpea-ui-backlink-preview-face
-         :no-decoration t
-         :on-click on-click
-         :help-echo nil))
-      (_
-       (vui-button (or (plist-get preview :text) "")
-         :face 'vulpea-ui-backlink-preview-face
-         :no-decoration t
-         :on-click on-click
-         :help-echo nil)))))
+    (let ((indicator (pcase type
+                       ('meta "⊢")
+                       ('header "§")
+                       ('table "▤")
+                       ('list "·")
+                       ('quote ">")
+                       ('code "λ")
+                       ('footnote "†")
+                       (_ nil)))
+          (text (pcase type
+                  ('meta (concat (plist-get preview :key) ": "
+                                 (or (plist-get preview :value) "")))
+                  (_ (or (plist-get preview :text) "")))))
+      (if indicator
+          (vui-hstack
+           :spacing 1
+           (vui-text indicator :face 'vulpea-ui-backlink-context-face)
+           (vui-button text
+             :face 'vulpea-ui-backlink-preview-face
+             :no-decoration t
+             :on-click on-click
+             :help-echo nil))
+        (vui-button text
+          :face 'vulpea-ui-backlink-preview-face
+          :no-decoration t
+          :on-click on-click
+          :help-echo nil)))))
 
 (defun vulpea-ui--jump-to-file-position (path pos)
   "Jump to position POS in file at PATH."
@@ -1359,7 +1338,7 @@ Renders the preview as a clickable button to jump to the mention."
                        (vui-text (format "(%d)" count)
                          :face 'vulpea-ui-widget-count-face))))
                   forward-links))
-              (vui-text "No links" :face 'shadow))))))))
+              (vui-muted "No links"))))))))
 
 (defun vulpea-ui--get-forward-links (note)
   "Get all notes linked from NOTE's file with counts.
@@ -1407,7 +1386,7 @@ Returns a list of plists with :note and :count, sorted by title."
            (seq-map (lambda (widget-sym)
                       (vui-component widget-sym :key widget-sym))
                     widgets)))
-      (vui-text "No vulpea note selected" :face 'shadow))))
+      (vui-muted "No vulpea note selected"))))
 
 (vui-defcomponent vulpea-ui-sidebar-root (note)
   "Root component for the sidebar with NOTE context."
@@ -1425,18 +1404,24 @@ Returns a list of plists with :note and :count, sorted by title."
          (buf-name (vulpea-ui--sidebar-buffer-name frame))
          (buf (get-buffer-create buf-name))
          (sidebar-win (vulpea-ui--get-sidebar-window frame))
-         (original-window (selected-window)))
+         (original-window (selected-window))
+         (existing-instance (gethash frame vulpea-ui--sidebar-instances)))
     ;; Select sidebar window before mount (vui-mount calls switch-to-buffer)
     (when sidebar-win
       (select-window sidebar-win t))
     (with-current-buffer buf
-      ;; Mount fresh - vui-mount calls kill-all-local-variables
-      (let ((new-instance
-             (vui-mount
-              (vui-component 'vulpea-ui-sidebar-root :note note)
-              buf-name)))
-        (puthash frame new-instance vulpea-ui--sidebar-instances))
-      ;; Set current note AFTER mount (vui-mount kills local variables)
+      (if (and existing-instance
+               (vui-instance-buffer existing-instance)
+               (buffer-live-p (vui-instance-buffer existing-instance)))
+          ;; Reuse existing instance - update props, preserve memos
+          (vui-update-props existing-instance (list :note note))
+        ;; Mount fresh - first render or instance was lost
+        (let ((new-instance
+               (vui-mount
+                (vui-component 'vulpea-ui-sidebar-root :note note)
+                buf-name)))
+          (puthash frame new-instance vulpea-ui--sidebar-instances)))
+      ;; Set current note AFTER render (vui-mount kills local variables)
       (setq vulpea-ui--current-note note)
       (goto-char (point-min)))
     ;; Restore original window
@@ -1496,14 +1481,19 @@ Returns a list of plists with :note and :count, sorted by title."
 
 ;;;###autoload
 (defun vulpea-ui-sidebar-refresh ()
-  "Force refresh the sidebar content."
+  "Force refresh the sidebar, invalidating all caches."
   (interactive)
   (let* ((frame (selected-frame))
          (main-win (vulpea-ui--get-main-window frame))
          (main-buf (when main-win (window-buffer main-win)))
-         (note (vulpea-ui--get-note-from-buffer main-buf)))
-    (setq vulpea-ui--current-note nil)  ; Force update
-    (vulpea-ui--render-sidebar note frame)))
+         (note (vulpea-ui--get-note-from-buffer main-buf))
+         (instance (gethash frame vulpea-ui--sidebar-instances)))
+    (when (and note instance
+               (vui-instance-buffer instance)
+               (buffer-live-p (vui-instance-buffer instance)))
+      (with-current-buffer (vui-instance-buffer instance)
+        (setq vulpea-ui--current-note note))
+      (vui-update instance (list :note note)))))
 
 
 ;;; Built-in widget registration
